severity = brutal
verbose = 8
exclude = Editor::RequireEmacsFileVariables Miscellanea::RequireRcsKeywords Modules::RequireExplicitInclusion Modules::RequireExplicitPackage Modules::RequirePerlVersion Tics::ProhibitLongLines BuiltInFunctions::ProhibitStringyEval

[CodeLayout::RequireTidyCode]
perltidyrc = xt/author/perltidy.rc

[Lax::RequireExplicitPackage::ExceptForPragmata]
allowed_pragmata = diagnostics feature perlversion strict warnings utf8 Modern::Perl

# since we're using Modern::Perl
[Compatibility::PerlMinimumVersionAndWhy]
above_version = 5.010
[Compatibility::PodMinimumVersion]
above_version = 5.010

[ControlStructures::ProhibitPostfixControls]
flowcontrol = carp cluck confess croak die exit goto warn exit

[Documentation::PodSpelling]
stop_words = Perldoc perldoc annonations PASSed DBIx accessor schemas DBICType TypeConstraints ResultSources namespace stringify DSN DBI dsn username plugins API configfile LoadingSchema nullable SQL

[Documentation::RequirePodSections]
lib_sections = NAME | VERSION | SYNOPSIS | DESCRIPTION | SUPPORT | AUTHOR | COPYRIGHT AND LICENSE
script_sections = NAME | USAGE | DESCRIPTION | CONFIGURATION | SUPPORT | AUTHOR | COPYRIGHT AND LICENSE

[InputOutput::RequireCheckedSyscalls]
functions = :builtins
exclude_functions = print say

[Subroutines::ProhibitCallsToUndeclaredSubs]
exempt_subs = Const::Fast::const Moose::extends Moose::has Moose::override Moose::with Moose::Role::around Moose::Role::has Moose::Role::requires MooseX::Has::Sugar::coerce MooseX::Has::Sugar::lazy MooseX::Has::Sugar::lazy_build MooseX::Has::Sugar::required MooseX::Has::Sugar::ro MooseX::Has::Sugar::rw MooseX::Types::as MooseX::Types::coerce MooseX::Types::from MooseX::Types::role_type MooseX::Types::subtype MooseX::Types::via Try::Tiny::catch Try::Tiny::try

[TestingAndDebugging::RequireUseStrict]
equivalent_modules = Modern::Perl

[TestingAndDebugging::RequireUseWarnings]
equivalent_modules = Modern::Perl
